unit BitBtnApi;
{

  ("`-''-/").___..--''"`-._
 `6_ 6  )   `-.  (     ).`-.__.`)
 (_Y_.)'  ._   )  `._ `. ``-..-'
  _..`--'_..-_/  /--'_.' ,'
(il).-''  (li).'  ((!.-'

 BitButton WinAPI Implementation

 Based on the source that freely available on the net
 Some copyright  2009 Denis Fateyev ( )
 Website: <http://www.fateyev.com>
 E-mail: <denis@fateyev.com>

 This program is free software; you can redistribute it and/or modify it under the terms
 of the GNU General Public License as published by the Free Software Foundation;
 either version 2 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with this program;
 if not, write to the Free Software Foundation, Inc.,
 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}

//-----------------------------------------------------
{$IFDEF FPC}
 {$MODE DELPHI}
{$ENDIF}
interface

uses Windows, Messages;

//-----------------------------------------------------
function Attach_BitBtnStyle( Wnd: HWND ): LRESULT;
function Detach_BitBtnStyle( Wnd: HWND ): LRESULT;
function BitBtn_Proc( Wnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM ): LRESULT; stdcall;
procedure BitBtn_Draw( hDrawIcon: HICON; DrawItem: PDrawItemStruct ); stdcall;

//-----------------------------------------------------
var
  lpOldBtn_Proc: Pointer = nil;

implementation

//-----------------------------------------------------
function Attach_BitBtnStyle( Wnd: HWND ): LRESULT;
begin
  lpOldBtn_Proc:= Pointer( SetWindowLong( Wnd, GWL_WNDPROC, Integer( @BitBtn_Proc ) ) );
  Result:= SetWindowLong( Wnd, GWL_STYLE, GetWindowLong( Wnd, GWL_STYLE ) + BS_OWNERDRAW );
end;

//-----------------------------------------------------
function Detach_BitBtnStyle( Wnd: HWND ): LRESULT;
begin
 Result:= 0;

 if ( lpOldBtn_Proc <> nil ) then
  Result:= SetWindowLong( Wnd, GWL_WNDPROC, LongInt( lpOldBtn_Proc ) );
end;

//-----------------------------------------------------
function BitBtn_Proc( Wnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM ): LRESULT; stdcall;
var
  lpPoint: TPoint; lpRect : TRect;

begin
  Result:= 0;

  case uMsg of
    // WM_MOSEMOVE
    WM_MOUSEMOVE:
      begin
        GetCursorPos( lpPoint );
        GetWindowRect( Wnd, lpRect );
        if ( PtInRect( lpRect, lpPoint ) ) then
          begin
            if ( GetCapture <> Wnd ) then
              begin
                SetCapture( Wnd );
                InvalidateRect( Wnd, nil, false );
              end;
          end
        else
          begin
            ReleaseCapture;
            InvalidateRect( Wnd, nil, false );
          end;
      end;
  else
    Result:= CallWindowProc( lpOldBtn_Proc, Wnd, uMsg, wParam, lParam );
  end;

end;

//-----------------------------------------------------
procedure BitBtn_Draw( hDrawIcon: HICON; DrawItem: PDrawItemStruct ); stdcall;
var
  lpBuffer: PWideChar;
  IconInfo: TIconInfo;
  OldStyle: DWORD;

begin
  FillRect( DrawItem.hdc, DrawItem.rcItem, HBRUSH( COLOR_BTNFACE + 1 ) );
  GetIconInfo( hDrawIcon, IconInfo );
  GetMem( lpBuffer, SizeOf( PWideChar) * MAX_PATH );

  try
    ZeroMemory( lpBuffer, SizeOf( PWideChar) * MAX_PATH );
    GetWindowTextW( DrawItem.hwndItem, lpBuffer, MAX_PATH - 1 );

    if ( ( ( DrawItem.itemState ) and ( ODS_SELECTED ) ) <> 0 ) then
       OldStyle:= DFCS_BUTTONPUSH + DFCS_PUSHED
    else
       OldStyle:= DFCS_BUTTONPUSH;

    if ( not( ( DrawItem.itemState and ODS_SELECTED ) <> 0 ) ) then
       OldStyle:= DFCS_BUTTONPUSH + DFCS_HOT;

    if ( ( DrawItem.itemState and ODS_DISABLED ) <> 0 ) then
       OldStyle:= DFCS_BUTTONPUSH + DFCS_INACTIVE;

    DrawFrameControl( DrawItem.hDC, DrawItem.rcItem, DFC_BUTTON, OldStyle );
    DrawItem.rcItem.Left  := DrawItem.rcItem.Left + Integer( IconInfo.xHotspot );
    DrawItem.rcItem.Right := DrawItem.rcItem.Right - Integer( IconInfo.xHotspot );

    if ( ( ( DrawItem.itemState ) and ( ODS_SELECTED ) ) <> 0 ) then
    begin
      Inc( DrawItem.rcItem.Left );
      Inc( DrawItem.rcItem.Top, 2 );
    end;

    DrawIconEx( DrawItem.hDC, DrawItem.rcItem.Left - 2,
      DrawItem.rcItem.Top + ( ( DrawItem.rcItem.Bottom - DrawItem.rcItem.Top ) div 2 ) - Integer( IconInfo.yHotspot ) - 1,
      hDrawIcon, IconInfo.xHotspot * 2, IconInfo.yHotspot * 2, 0, 0, DI_NORMAL );

    DrawItem.rcItem.Left  := DrawItem.rcItem.Left - Integer( IconInfo.xHotspot );
    DrawItem.rcItem.Right := DrawItem.rcItem.Right + Integer( IconInfo.xHotspot );
    DrawItem.rcItem.Left  := IconInfo.xHotspot + IconInfo.xHotspot * 2;
    DrawItem.rcItem.Right := DrawItem.rcItem.Right - Integer( IconInfo.xHotspot );

    Dec( DrawItem.rcItem.Top );
    if ( ( ( DrawItem.itemState ) and ( ODS_SELECTED ) ) <> 0 ) then
    begin
      Inc( DrawItem.rcItem.Left );
      Inc( DrawItem.rcItem.Top );
    end;

    DrawTextW( DrawItem.hDC, lpBuffer, -1, DrawItem.rcItem, DT_SINGLELINE + DT_NOPREFIX + DT_VCENTER + DT_CENTER );

    if ( IconInfo.hbmMask <> 0 ) then DeleteObject( IconInfo.hbmMask );
    if ( IconInfo.hbmColor <> 0 ) then DeleteObject( IconInfo.hbmColor );

  finally
    FreeMem( lpBuffer );
  end;

end;

//-----------------------------------------------------

end.
