unit helper_MiscUtils;
{

  ("`-''-/").___..--''"`-._
 `6_ 6  )   `-.  (     ).`-.__.`)
 (_Y_.)'  ._   )  `._ `. ``-..-'
  _..`--'_..-_/  /--'_.' ,'
(il).-''  (li).'  ((!.-'

 Terminal Session Helper (Misc Utils Unit)

 Copyright  2008-2010 Denis Fateyev ( )
   Website: <http://www.fateyev.com>
   E-Mail: <denis@fateyev.com>

 KOL (Key Object Library)
    Copyright  1999-2008 Vladimir Kladov and KOL developers
      Website: <http://kolmck.net>

 LENIN INC WIN32API Library v1.1 (build 16.08.05)

    Copyright  Drigalkin Vladimir, LENIN INC.
      Website: <http://www.lenininc.com>

 Terminal Services API interface Unit for Object Pascal

    Portions created by Microsoft Copyright  1995-2001 Microsoft Corporation.
      Website: <http://www.microsoft.com>

    Copyright  1999-2001 Marcel van Brakel (Project JEDI)
      Website: <http://members.chello.nl/m.vanbrakel2>, <http://delphi-jedi.org>


 This program is free software; you can redistribute it and/or modify it under the terms
 of the GNU General Public License as published by the Free Software Foundation;
 either version 2 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with this program;
 if not, write to the Free Software Foundation, Inc.,
 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}

//-----------------------------------------------------
{$IFDEF FPC}
 {$MODE DELPHI}
{$ENDIF}
interface

uses Windows;

//-----------------------------------------------------
function CompareMem( P1, P2: Pointer; Length: Integer ): Boolean;
function ExtractFilePath( const Path : string ) : string;
function __DelimiterLast( Str, Delimiters: PChar ): PChar;
function StrLen( const Str: PChar ): Cardinal;
function StrRScan( const Str: PChar; Chr: Char ): PChar;
function FileExists( const FileName : string ) : Boolean;
function Parse( var S : string; const Separators : string ) : string;
function StrScan( Str: PChar; Chr: Char ): PChar;
function IndexOfCharsMin( const S, Chars : string ) : Integer;
function IndexOfChar( const S : string; Chr : Char ) : Integer;
function LoggedOnUserNameEx( fFormat: DWORD ): string;
function SetDebugPrivileges: Boolean;
function SIDToUsername( SID: PSID; var strUsername: string ): Boolean;
function GetProcessTimeCreate( dwProcessID: DWord; var timeCreate: PFileTime ): Boolean;

// NOTE: IniReadString() reads maximum only 1024 chars (bytes) from the parameter,
// so if you want more please adjust 'Buffer' variable in the function's body
function IniReadString( const FFileName, Section, Ident, Default: string ): string;
function IniReadInteger( const FFileName, Section, Ident: string; Default: Integer ): Integer;

function Int2Str( Value: Integer ): string;
function Str2Int( const Value : string ) : Integer;
function DateTimeToStr( pst: PSystemTime ): string;
function DateTimeNowToStr: string;
function FileTimeToStr( pft: PFileTime ): string;
function FillZeroDigits( Value: Integer ): string;
function AddLogFileLine( strLogFile: string; strLine: string ): Boolean;

// this requres Windows version not less than Windows 2000
function GetUserNameEx( NameFormat: DWORD; lpNameBuffer: LPSTR; nSize: PULONG ): BOOL; stdcall;
 external 'secur32.dll' name 'GetUserNameExA';
{$EXTERNALSYM GetUserNameEx}

//-----------------------------------------------------
{$IFNDEF FPC}
type
  // add missing TOKEN_USER declaration
  PTOKEN_USER = ^TOKEN_USER;
  TOKEN_USER = record
    User : SID_AND_ATTRIBUTES;
  end;
{$ENDIF}
type
  // session connect state
  _WTS_CONNECTSTATE_CLASS = (
    WTSActive,              // User logged on to WinStation
    WTSConnected,           // WinStation connected to client
    WTSConnectQuery,        // In the process of connecting to client
    WTSShadow,              // Shadowing another WinStation
    WTSDisconnected,        // WinStation logged on without client
    WTSIdle,                // Waiting for client to connect
    WTSListen,              // WinStation is listening for connection
    WTSReset,               // WinStation is being reset
    WTSDown,                // WinStation is down due to error
    WTSInit );              // WinStation in initialization
  {$EXTERNALSYM _WTS_CONNECTSTATE_CLASS}
  WTS_CONNECTSTATE_CLASS = _WTS_CONNECTSTATE_CLASS;
  {$EXTERNALSYM WTS_CONNECTSTATE_CLASS}
  TWtsConnectStateClass = WTS_CONNECTSTATE_CLASS;

//  WTSEnumerateSessions() returns data in a similar format to the above
//  WTSEnumerateServers().  It returns two variables: pSessionInfo and
//  Count.  The latter is the number of WTS_SESSION_INFO structures
//  contained in the former.  Iteration is similar, except that there
//  are three parts to each entry, so it would look like this:
//
//  for ( i=0; i < Count; i++ ) {
//      _tprintf( TEXT("%-5u  %-20s  %u\n"),
//                pSessionInfo[i].SessionId,
//                pSessionInfo[i].pWinStationName,
//                pSessionInfo[i].State );
//  }
//
//  The memory returned is also segmented as the above, with all the
//  structures allocated at the start and the string data at the end.
//  We'll use S for the SessionId, P for the pWinStationName pointer
//  and D for the string data, and C for the connect State:
//
//  S1 P1 C1 S2 P2 C2 S3 P3 C3 S4 P4 C4 ... Sn Pn Cn D1 D2 D3 D4 ... Dn
//
//  As above, this makes it easier to iterate the sessions.
type
  PWTS_SESSION_INFOA = ^WTS_SESSION_INFOA;
  {$EXTERNALSYM PWTS_SESSION_INFOA}
  _WTS_SESSION_INFOA = record
    SessionId: DWORD;              // session id
    pWinStationName: LPSTR;        // name of WinStation this session is connected to
    State: WTS_CONNECTSTATE_CLASS; // connection state (see enum)
  end;
  {$EXTERNALSYM _WTS_SESSION_INFOA}
  WTS_SESSION_INFOA = _WTS_SESSION_INFOA;
  {$EXTERNALSYM WTS_SESSION_INFOA}
  TWtsSessionInfoA = WTS_SESSION_INFOA;
  PWtsSessionInfoA = PWTS_SESSION_INFOA;

  WTS_SESSION_INFO = WTS_SESSION_INFOA;
  PWTS_SESSION_INFO = PWTS_SESSION_INFOA;
  TWtsSessionInfo = TWtsSessionInfoA;
  PWtsSessionInfo = PWtsSessionInfoA;

  //  WTSEnumerateProcesses() also returns data similar to
  //  WTSEnumerateServers().  It returns two variables: pProcessInfo and
  //  Count.  The latter is the number of WTS_PROCESS_INFO structures
  //  contained in the former.  Iteration is similar, except that there
  //  are four parts to each entry, so it would look like this:
  //
  //  for ( i=0; i < Count; i++ ) {
  //      GetUserNameFromSid( pProcessInfo[i].pUserSid, UserName,
  //                          sizeof(UserName) );
  //      _tprintf( TEXT("%-5u  %-20s  %-5u  %s\n"),
  //              pProcessInfo[i].SessionId,
  //              UserName,
  //              pProcessInfo[i].ProcessId,
  //              pProcessInfo[i].pProcessName );
  //  }
  //
  //  The memory returned is also segmented as the above, with all the
  //  structures allocated at the start and the string data at the end.
  //  We'll use S for the SessionId, R for the ProcessId, P for the
  //  pProcessName pointer and D for the string data, and U for pUserSid:
  //
  //  S1 R1 P1 U1 S2 R2 P2 U2 S3 R3 P3 U3 ... Sn Rn Pn Un D1 D2 D3 ... Dn
  //
  //  As above, this makes it easier to iterate the processes.
  //
type
  PWTS_PROCESS_INFOA = ^WTS_PROCESS_INFOA;
  {$EXTERNALSYM PWTS_PROCESS_INFOA}
  _WTS_PROCESS_INFOA = record
    SessionId: DWORD;    // session id
    ProcessId: DWORD;    // process id
    pProcessName: LPSTR; // name of process
    pUserSid: PSID;      // user's SID
  end;
  {$EXTERNALSYM _WTS_PROCESS_INFOA}
  WTS_PROCESS_INFOA = _WTS_PROCESS_INFOA;
  {$EXTERNALSYM WTS_PROCESS_INFOA}
  TWtsProcessInfoA = WTS_PROCESS_INFOA;
  PWtsProcessInfoA = PWTS_PROCESS_INFOA;

  WTS_PROCESS_INFO = WTS_PROCESS_INFOA;
  {$EXTERNALSYM WTS_PROCESS_INFO}
  PWTS_PROCESS_INFO = PWTS_PROCESS_INFOA;
  {$EXTERNALSYM PWTS_PROCESS_INFO}
  TWtsProcessInfo = TWtsProcessInfoA;
  PWtsProcessInfo = PWtsProcessInfoA;

function WTSEnumerateSessions( hServer: THandle; Reserved: DWORD; Version: DWORD;
 var ppSessionInfo: PWTS_SESSION_INFOA; var pCount: DWORD ): BOOL; stdcall;
  external 'wtsapi32.dll' name 'WTSEnumerateSessionsA';
{$EXTERNALSYM WTSEnumerateSessions}

function WTSEnumerateProcesses( hServer: THandle; Reserved: DWORD; Version: DWORD;
 var ppProcessInfo: PWTS_PROCESS_INFOA; var pCount: DWORD ): BOOL; stdcall;
  external 'wtsapi32.dll' name 'WTSEnumerateProcessesA';
{$EXTERNALSYM WTSEnumerateProcesses}

procedure WTSFreeMemory( pMemory: Pointer ); stdcall;
  external 'wtsapi32.dll' name 'WTSFreeMemory';
{$EXTERNALSYM WTSFreeMemory}

const
  dwNameUnknown           = 0;  // Unknown name type.
  dwNameFullyQualifiedDN  = 1;  // Fully qualified distinguished name
  dwNameSamCompatible     = 2;  // Windows NT 4.0 account name
  dwNameDisplay           = 3;  // A "friendly" display name
  dwNameUniqueId          = 6;  // GUID string that the IIDFromString function returns
  dwNameCanonical         = 7;  // Complete canonical name
  dwNameUserPrincipal     = 8;  // User principal name
  dwNameCanonicalEx       = 9;
  dwNameServicePrincipal  = 10; // Generalized service principal name
  dwDNSDomainName         = 11; // DNS domain name, plus the user name

  arMonthNames: array[0..11] of string =
   ( 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec' );

  // debug privilege alias (especially for Delphi)
  {$IFNDEF FPC}
  SE_DEBUG_NAME = 'SeDebugPrivilege';
  {$ENDIF}

  // specifies the current server
  WTS_CURRENT_SERVER        = THandle( 0 );
  {$EXTERNALSYM WTS_CURRENT_SERVER}
  WTS_CURRENT_SERVER_HANDLE = THandle( 0 );
  {$EXTERNALSYM WTS_CURRENT_SERVER_HANDLE}
  WTS_CURRENT_SERVER_NAME   = '';
  {$EXTERNALSYM WTS_CURRENT_SERVER_NAME}

// ----------------------------------------------------------
var
  strUsername: array[0..MAX_PATH - 1] of Char;
  strDomain:   array[0..MAX_PATH - 1] of Char;

implementation

//-----------------------------------------------------
function CompareMem( P1, P2: Pointer; Length: Integer ): Boolean; assembler;
asm
  {$IFDEF F_P}
        MOV     EAX, [P1]
        MOV     EDX, [P2]
        MOV     ECX, [Length]
  {$ENDIF}
        PUSH    ESI
        PUSH    EDI
        MOV     ESI,P1
        MOV     EDI,P2
        MOV     EDX,ECX
        XOR     EAX,EAX
        AND     EDX,3
        SHR     ECX,1
        SHR     ECX,1
        REPE    CMPSD
        JNE     @@2
        MOV     ECX,EDX
        REPE    CMPSB
        JNE     @@2
@@1:    INC     EAX
@@2:    POP     EDI
        POP     ESI
end {$IFDEF F_P} [ 'EAX', 'EDX', 'ECX' ] {$ENDIF};

//-----------------------------------------------------
function Int2Str( Value: Integer ): string;
var Buf : array[ 0..15 ] of Char;
    Dst : PChar;
    Minus : Boolean;
    D : DWORD;
begin
  Dst := @Buf[ 15 ];
  Dst^ := #0;
  Minus := False;
  if Value < 0 then
  begin
    Value := -Value;
    Minus := True;
  end;
  D := Value;
  repeat
    Dec( Dst );
    Dst^ := Char( (D mod 10) + Byte( '0' ) );
    D := D div 10;
  until D = 0;
  if Minus then
  begin
    Dec( Dst );
    Dst^ := '-';
  end;
  Result := Dst;
end;

//-----------------------------------------------------
function LoggedOnUserNameEx( fFormat: DWORD ): string;
var
  szUserName: array[0..MAX_PATH] of Char;
  Size: DWORD;

begin
  Size := MAX_PATH - 1;
  GetUserNameEx( fFormat, @szUserName, @Size );

  Result := szUserName;
end;

//-----------------------------------------------------
function SetDebugPrivileges: Boolean;
var
  hToken : THandle;
  tkp, tkpOld : TOKEN_PRIVILEGES;
  dwReturnLength : DWORD;

begin
 Result:= false;

  if ( OpenProcessToken( GetCurrentProcess, TOKEN_ADJUST_PRIVILEGES + TOKEN_QUERY, hToken ) ) then
   if ( LookupPrivilegeValue( nil, SE_DEBUG_NAME, tkp.Privileges[0].Luid ) ) then
    begin
      dwReturnLength := 0;
      tkp.PrivilegeCount := 1;
      tkp.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
      AdjustTokenPrivileges( hToken, false, tkp, SizeOf( TTokenPrivileges ), tkpOld , dwReturnLength );
      if ( GetLastError = ERROR_SUCCESS ) then
        Result:= true;
    end;
end;

//-----------------------------------------------------
function SIDToUsername( SID: PSID; var strUsername: string ): Boolean;
var
  pUsername, pDomain: PChar;
  dwLen1, dwLen2: DWord;
  peUse: SID_NAME_USE;

begin
  Result:= false;

  if ( IsValidSid( SID ) ) then
  begin
    dwLen1:= MAX_PATH - 1;
    dwLen2:= MAX_PATH - 1;

    GetMem( pUsername, MAX_PATH );
    GetMem( pDomain, MAX_PATH );

    try
      if ( LookupAccountSid( nil, SID, pUsername, dwLen1, pDomain, dwLen2, peUse ) ) then
      begin;
        strUsername:= Copy( pDomain, 0, lstrlen( pDomain ) ) + '\' + Copy( pUsername, 0, lstrlen( pUsername ) );
        Result:= true;
      end;

    finally
      FreeMem( pDomain );
      FreeMem( pUsername );
    end;

  end; // 'if IsValidSid( ptiUser^.User.Sid ) then'

end;

//-----------------------------------------------------
function GetProcessTimeCreate( dwProcessID: DWord; var timeCreate: PFileTime ): Boolean;
var
  hProcess: THandle;
  timeExit, timeKernel, timeUser: FILETIME;

begin
  Result:= false;

  hProcess:= OpenProcess( PROCESS_QUERY_INFORMATION + PROCESS_VM_READ, false, dwProcessId );
  if ( hProcess > 0 ) then
  begin
    if ( GetProcessTimes( hProcess, timeCreate^, timeExit, timeKernel, timeUser ) ) then
      Result:= true;

    CloseHandle( hProcess );
  end;
end;

//-----------------------------------------------------
function ExtractFilePath( const Path : string ) : string;
var P, P0: PChar;
begin
  P0 := PChar( Path );
  P := __DelimiterLast( P0, ':\/' );
  if P^ = #0 then
    Result := ''
  else
    Result := Copy( Path, 1, P - P0 + 1 );
end;

//-----------------------------------------------------
function __DelimiterLast( Str, Delimiters: PChar ): PChar;
var
    P, F : PChar;
begin
  P := Str;
  Result := P + StrLen ( Str );
  while Delimiters^ <> #0 do
  begin
    F := StrRScan
         ( P, Delimiters^ );
    if F <> nil then
    if (Result^ = #0) or (Integer(F) > Integer(Result)) then
       Result := F;
    Inc( Delimiters );
  end;
end;

//-----------------------------------------------------
function StrLen(const Str: PChar): Cardinal; assembler;
asm
  {$IFDEF F_P}
        MOV     EAX, [Str]
  {$ENDIF F_P}
        XCHG    EAX, EDI
        XCHG    EDX, EAX
        OR      ECX, -1
        XOR     EAX, EAX
        CMP     EAX, EDI
        JE      @@exit0
        REPNE   SCASB
        DEC     EAX
        DEC     EAX
        SUB     EAX,ECX
@@exit0:
        MOV     EDI,EDX
end {$IFDEF F_P} [ 'EAX', 'EDX', 'ECX' ] {$ENDIF};

//-----------------------------------------------------
function StrRScan(const Str: PChar; Chr: Char): PChar; assembler;
asm
  {$IFDEF F_P}
        MOV     EAX, [Str]
        MOVZX   EDX, [Chr]
  {$ENDIF F_P}
        PUSH    EDI
        MOV     EDI,Str
        MOV     ECX,0FFFFFFFFH
        XOR     AL,AL
        REPNE   SCASB
        NOT     ECX
        STD
        DEC     EDI
        MOV     AL,Chr
        REPNE   SCASB
        MOV     EAX,0
        JNE     @@1
        MOV     EAX,EDI
        INC     EAX
@@1:    CLD
        POP     EDI
end {$IFDEF F_P} [ 'EAX', 'EDX', 'ECX' ] {$ENDIF};

//-----------------------------------------------------
function FileExists( const FileName : string ) : Boolean;
var Code: Integer;
begin
  Code := GetFileAttributes( PChar( FileName ) );
  Result := ( Code <> -1 ) and ( FILE_ATTRIBUTE_DIRECTORY and Code = 0 );
end;

//-----------------------------------------------------
function Parse( var S : string; const Separators : string ) : string;
var Pos : Integer;
begin
  Pos := IndexOfCharsMin( S, Separators );
  if Pos <= 0 then
     Pos := Length( S )+1;
  Result := Copy( S, 1, Pos-1 );
  Delete( S, 1, Pos );
end;

//-----------------------------------------------------
function StrScan(Str: PChar; Chr: Char): PChar; assembler;
asm
  {$IFDEF F_P}
  MOV   EAX, [Str]
  MOVZX EDX, [Chr]
  {$ENDIF}
        PUSH    EDI
        PUSH    EAX
        MOV     EDI,Str
        OR      ECX, -1
        XOR     AL,AL
        REPNE   SCASB
        NOT     ECX
        POP     EDI
        XCHG    EAX, EDX
        REPNE   SCASB

        XCHG    EAX, EDI
        POP     EDI

        JE      @@1
        XOR     EAX, EAX
        RET

@@1:    DEC     EAX
end {$IFDEF F_P} [ 'EAX', 'EDX', 'ECX' ] {$ENDIF};

//-----------------------------------------------------
function IndexOfCharsMin( const S, Chars : string ) : Integer;
var I, J : Integer;
begin
  Result := -1;
  for I := 1 to Length( Chars ) do
  begin
    J := IndexOfChar( S, Chars[ I ] );
    if J > 0 then
    begin
      if (Result <= 0) or (J < Result) then
         Result := J;
    end;
  end;
end;

//-----------------------------------------------------
function IndexOfChar( const S : string; Chr : Char ) : Integer;
var P, F : PChar;
begin
   P := PChar( S );
   F := StrScan( P, Chr );
   Result := -1;
   if F = nil then Exit;
   Result := Integer( F ) - Integer( P ) + 1;
end;

//-----------------------------------------------------
function S2Int( S: PAnsiChar ): Integer;
var M : Integer;
begin
   Result := 0;
   if S = '' then Exit;
   M := 1;
   if S^ = '-' then
   begin
      M := -1;
      Inc( S );
   end
     else
   if S^ = '+' then
     Inc( S );
   while S^ in [ '0'..'9' ] do
   begin
      Result := Result * 10 + Integer( S^ ) - Integer( '0' );
      Inc( S );
   end;
   if M < 0 then
      Result := -Result;
end;

//-----------------------------------------------------
function Str2Int( const Value : string ) : Integer;
begin
  Result := S2Int( PChar( Value ) );
end;

//-----------------------------------------------------
function IniReadString( const FFileName, Section, Ident, Default: string ): string;
var
  Buffer: array[0..1023] of Char;
begin
  SetString( Result, Buffer, GetPrivateProfileString( PChar( Section ),
   PChar( Ident ), PChar( Default ), Buffer, SizeOf( Buffer ), PChar( FFileName ) ) );
end;

//-----------------------------------------------------
function IniReadInteger( const FFileName, Section, Ident: string; Default: Integer ): Integer;
var
 strIntValue: string;
begin
 Result:= Default;

 strIntValue:= IniReadString( PChar( FFileName ), Section, Ident, '' );
 if ( strIntValue <> '' ) then
 begin
   Result:= Str2Int( strIntValue );
   if ( Result = 0 ) then Result:= Default;
 end;
 
end;

//-----------------------------------------------------
function FillZeroDigits( Value: Integer ): string;
begin
  Result:= Int2Str( Value );
  if ( Value < 10 ) then Result:= '0' + Result;
end;

// ----------------------------------------------------------
function DateTimeToStr( pst: PSystemTime ): string;
begin
  with ( pst^ ) do
   Result:= FillZeroDigits( wDay ) + ' ' + arMonthNames[ wMonth - 1 ] + ' ' + Int2Str( wYear ) +
    ', ' + FillZeroDigits( wHour ) + ':' + FillZeroDigits( wMinute ) + ':' + FillZeroDigits( wSecond );
end;

// ----------------------------------------------------------
function DateTimeNowToStr: string;
var
  st: SYSTEMTIME;
begin
  GetLocalTime( st );
  Result:= DateTimeToStr( @st );
end;

// ----------------------------------------------------------
function FileTimeToStr( pft: PFileTime ): string;
var
  st: SYSTEMTIME; ft: FILETIME;
begin
  FileTimeToLocalFileTime( pft^, ft );
  FileTimeToSystemTime( ft, st );
  Result:= DateTimeToStr( @st );
end;

// ----------------------------------------------------------
function AddLogFileLine( strLogFile: string; strLine: string ): Boolean;
var
  hLogFile: THandle;
  strBuffer: string;
  bWritten: Integer;
begin
  Result:= false;

  hLogFile:= CreateFile( PChar( strLogFile ), GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ + FILE_SHARE_WRITE,
    nil, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0 );
  if ( hLogFile = INVALID_HANDLE_VALUE ) then Exit;

  SetFilePointer( hLogFile, 0, nil, FILE_END );
  strBuffer:= DateTimeNowToStr + ' (' + LoggedOnUserNameEx( dwNameSamCompatible ) + '): ' +
   {$IFDEF FPC} UTF8ToAnsi( strLine ) {$ELSE} strLine {$ENDIF} + #13#10;
  WriteFile( hLogFile, PChar( strBuffer )^, Length( strBuffer ), Cardinal( bWritten ), nil );
  CloseHandle( hLogFile );

  if ( bWritten = Length( strBuffer ) ) then Result:= true;
end;

// ----------------------------------------------------------


end.
